@@ -1,8 +1,7 @@
 #ifndef MODULE_H
 #define MODULE_H
 
-struct queue_buff_struct
-{
+struct queue_buff_struct {
   struct queue_buff_struct *next;
   unsigned long *data;
   int size;
@@ -38,9 +37,11 @@ extern struct queue_buff_struct **pipeQueue;
 #define M_END_CONFIG_INFO    (1<<19)
 #define M_ICON_FILE          (1<<20)
 #define M_DEFAULTICON        (1<<21)
-#define M_FUNCTION_END       (1<<22)
-#define M_TITLE_ICON         (1<<23)
-#define MAX_MESSAGES         24
+#define M_STRING             (1<<22)
+#define M_FUNCTION_END       (1<<23)
+#define M_MINI_ICON          (1<<24)
+#define M_SCROLLREGION       (1<<25)
+#define MAX_MESSAGES         26
 #define MAX_MASK             ((1<<MAX_MESSAGES)-1)
 
 #define HEADER_SIZE         4
@@ -48,7 +49,4 @@ extern struct queue_buff_struct **pipeQueue;
 #define MAX_PACKET_SIZE    (HEADER_SIZE+MAX_BODY_SIZE)
 
 
-#endif /* MODULE_H */
-
-
-
+#endif				/* MODULE_H */
