@@ -13,14 +13,11 @@
 #define TRUE 1
 #define FALSE 0
 
-#define MARGIN1 8
-#define MARGIN2 6
-
 #define NONE 0
 #define VERTICAL 1
 #define HORIZONTAL 2
 
-#include "../../configure.h"
+#include <FVWMconfig.h>
 
 #ifdef ISC
 #include <sys/bsdtypes.h> /* Saul */
@@ -56,7 +53,7 @@
 #endif
 
 #include "FvwmIconBox.h"
-#include "../../version.h"
+#include <fvwm/version.h>
 
 char *MyName;
 
@@ -131,6 +128,8 @@ int ClickTime = 150;
 unsigned int bar_width = 9;
 int redraw_flag = 3;
 int icon_relief = 4;
+int margin1 = 8;
+int margin2 = 6;
 
 Pixmap IconwinPixmap = None;
 char *IconwinPixmapFile = NULL;
@@ -164,7 +163,7 @@ int sortby = UNSORT;
   Based on main() from GoodStuff:
   	Copyright 1993, Robert Nation.
 ************************************************************************/
-void main(int argc, char **argv)
+int main(int argc, char **argv)
 {
   char *display_name = NULL;
   char *temp, *s;
@@ -230,6 +229,8 @@ void main(int argc, char **argv)
   SendFvwmPipe(fd,"Send_WindowList",0);
 
   Loop();
+
+  return 0;
 }
 
 /************************************************************************
@@ -288,8 +289,8 @@ void Loop(void)
 		num_rows = (th - v_margin - interval + 1) / UHeight;
 		Width = UWidth * num_columns + interval - 1; 
 		Height = UHeight * num_rows + interval -1; 
-		XMoveResizeWindow(dpy, holder_win, MARGIN1+2,
-				  MARGIN1+2,   
+		XMoveResizeWindow(dpy, holder_win, margin1+2,
+				  margin1+2,   
 				  tw - h_margin, th - v_margin);
 		if (!(local_flags & HIDE_H))
 		  XResizeWindow(dpy, h_scroll_bar, 
@@ -543,13 +544,13 @@ void RedrawWindow(void)
 
   while (XCheckTypedWindowEvent (dpy, main_win, Expose, &dummy));
 
-  RelieveWindow(main_win, MARGIN1, MARGIN1, Width + 4, 
+  RelieveWindow(main_win, margin1, margin1, Width + 4, 
 		Height + 4, ShadowGC,ReliefGC);   
   if (!(local_flags & HIDE_H))
-    RelieveWindow(main_win, MARGIN1, MARGIN1 + 4 + Height + MARGIN2, 
+    RelieveWindow(main_win, margin1, margin1 + 4 + Height + margin2, 
 		  Width + 4, bar_width+4, ShadowGC,ReliefGC);   
   if (!(local_flags & HIDE_V))
-    RelieveWindow(main_win, MARGIN1 + 4 + Width + MARGIN2, MARGIN1, 
+    RelieveWindow(main_win, margin1 + 4 + Width + margin2, margin1, 
 		  bar_width+4, Height + 4, ShadowGC,ReliefGC);
   RelieveWindow(main_win, 0, 0, Width + h_margin, Height + v_margin, 
 		ReliefGC, ShadowGC);   
@@ -828,7 +829,7 @@ void RedrawBottomButton(GC rgc, GC sgc)
     Original work from GoodStuff:
       Copyright 1993, Robert Nation.
 ************************************************************************/
-FVWM_INLINE void RelieveWindow(Window win,int x,int y,int w,int h,
+inline void RelieveWindow(Window win,int x,int y,int w,int h,
 		   GC rgc,GC sgc)
 {
   XSegment seg[4];
@@ -882,8 +883,8 @@ void CreateWindow(void)
   unsigned long gcm;
   unsigned long mask;
 
-  h_margin = MARGIN1*2 + bar_width + MARGIN2 + 8;
-  v_margin = MARGIN1*2 + bar_width + MARGIN2 + 8;
+  h_margin = margin1*2 + bar_width + margin2 + 8;
+  v_margin = margin1*2 + bar_width + margin2 + 8;
 
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   _XA_WM_PROTOCOLS = XInternAtom (dpy, "WM_PROTOCOLS", False);
@@ -899,9 +900,9 @@ void CreateWindow(void)
     };
 
   if ((local_flags & HIDE_H))
-    v_margin -= bar_width + MARGIN2 + 4;
+    v_margin -= bar_width + margin2 + 4;
   if ((local_flags & HIDE_V))
-    h_margin -= bar_width + MARGIN2 + 4;
+    h_margin -= bar_width + margin2 + 4;
 
   UWidth = max_icon_width + icon_relief + interval;
   UHeight = font->ascent + font->descent + max_icon_height +
@@ -978,8 +979,8 @@ void CreateWindow(void)
   mysizehints.flags = PWinGravity| PResizeInc | PSize;
   mysizehints.width -= h_margin;
   mysizehints.height -= v_margin;
-  holder_win = XCreateSimpleWindow(dpy,main_win,MARGIN1+2,
-				   MARGIN1+2,   
+  holder_win = XCreateSimpleWindow(dpy,main_win,margin1+2,
+				   margin1+2,   
 				   mysizehints.width,
 				   mysizehints.height, 
                                    0,fore_pix,back_pix);
@@ -1032,9 +1033,9 @@ void CreateWindow(void)
   attributes.background_pixel = back_pix;
   if (!(local_flags & HIDE_H)){
     attributes.win_gravity = SouthWestGravity;
-    h_scroll_bar = XCreateWindow(dpy ,main_win, MARGIN1 + 2 +
+    h_scroll_bar = XCreateWindow(dpy ,main_win, margin1 + 2 +
 				 bar_width, 
-				 MARGIN1 + 6 + Height + MARGIN2,
+				 margin1 + 6 + Height + margin2,
 				 Width - bar_width*2, bar_width,
 				 0, CopyFromParent,
 				 InputOutput, CopyFromParent, 
@@ -1043,9 +1044,9 @@ void CreateWindow(void)
   }
   if (!(local_flags & HIDE_V)){
     attributes.win_gravity = NorthEastGravity;
-    v_scroll_bar = XCreateWindow(dpy ,main_win, MARGIN1 + 6 +
-				 Width + MARGIN2, 
-				 MARGIN1 + 2 + bar_width,
+    v_scroll_bar = XCreateWindow(dpy ,main_win, margin1 + 6 +
+				 Width + margin2, 
+				 margin1 + 2 + bar_width,
 				 bar_width, Height - bar_width*2, 
 				 0, CopyFromParent, 
 				 InputOutput, CopyFromParent,
@@ -1056,16 +1057,16 @@ void CreateWindow(void)
   /* buttons */
   if (!(local_flags & HIDE_H)){
     attributes.win_gravity = SouthWestGravity;
-    l_button = XCreateWindow(dpy, main_win, MARGIN1 + 2,
-			     MARGIN1 + 6 + Height + MARGIN2,
+    l_button = XCreateWindow(dpy, main_win, margin1 + 2,
+			     margin1 + 6 + Height + margin2,
 			     bar_width, bar_width,
 			     0, CopyFromParent, 
 			     InputOutput, CopyFromParent,
 			     mask, &attributes);
     attributes.win_gravity = SouthEastGravity;
-    r_button = XCreateWindow(dpy, main_win, MARGIN1 + 2 + Width -
+    r_button = XCreateWindow(dpy, main_win, margin1 + 2 + Width -
 			     bar_width, 
-			     MARGIN1 + 6 + Height + MARGIN2,
+			     margin1 + 6 + Height + margin2,
 			     bar_width, bar_width,
 			     0, CopyFromParent, 
 			     InputOutput, CopyFromParent,
@@ -1075,16 +1076,16 @@ void CreateWindow(void)
   }
   if (!(local_flags & HIDE_V)){
     attributes.win_gravity = NorthEastGravity;
-    t_button = XCreateWindow(dpy, main_win, MARGIN1 + 6 +
-			     Width + MARGIN2, MARGIN1 + 2,
+    t_button = XCreateWindow(dpy, main_win, margin1 + 6 +
+			     Width + margin2, margin1 + 2,
 			     bar_width, bar_width,
 			     0, CopyFromParent, 
 			     InputOutput, CopyFromParent,
 			     mask, &attributes);
     attributes.win_gravity = SouthEastGravity;
-    b_button = XCreateWindow(dpy, main_win, MARGIN1 + 6 +
-			     Width + MARGIN2,
-			     MARGIN1 + 2 + Height - bar_width,
+    b_button = XCreateWindow(dpy, main_win, margin1 + 6 +
+			     Width + margin2,
+			     margin1 + 2 + Height - bar_width,
 			     bar_width, bar_width,
 			     0, CopyFromParent, 
 			     InputOutput, CopyFromParent,
@@ -1128,9 +1129,9 @@ Pixel GetShadow(Pixel background)
   bg_color.pixel = background;
   XQueryColor(dpy,attributes.colormap,&bg_color);
   
-  bg_color.red = (unsigned short)((bg_color.red*60)/100);
-  bg_color.green = (unsigned short)((bg_color.green*60)/100);
-  bg_color.blue = (unsigned short)((bg_color.blue*60)/100);
+  bg_color.red = (unsigned short)((bg_color.red*50)/100);
+  bg_color.green = (unsigned short)((bg_color.green*50)/100);
+  bg_color.blue = (unsigned short)((bg_color.blue*50)/100);
   
   if(!XAllocColor(dpy,attributes.colormap,&bg_color))
     nocolor("alloc shadow","");
@@ -1245,22 +1246,22 @@ void SendFvwmPipe(int *fd, char *message,unsigned long window)
 
 Bool ExecIconBoxFunction(char *msg)
 {
-  if (mystrncasecmp(msg, "Next", 4) == 0){
+  if (strncasecmp(msg, "Next", 4) == 0){
     Next();
     return True;
-  }else if (mystrncasecmp(msg, "Prev", 4) == 0){
+  }else if (strncasecmp(msg, "Prev", 4) == 0){
     Prev();
     return True;
-  }else if (mystrncasecmp(msg, "Left", 4) == 0){
+  }else if (strncasecmp(msg, "Left", 4) == 0){
     HScroll(icon_win_x - UWidth); 
     return True;
-  }else if (mystrncasecmp(msg, "Right", 5) == 0){
+  }else if (strncasecmp(msg, "Right", 5) == 0){
     HScroll(icon_win_x + UWidth);
     return True;
-  }else if (mystrncasecmp(msg, "Up", 2) == 0){
+  }else if (strncasecmp(msg, "Up", 2) == 0){
     VScroll(icon_win_y - UHeight);
     return True;
-  }else if (mystrncasecmp(msg, "Down", 4) == 0){
+  }else if (strncasecmp(msg, "Down", 4) == 0){
     VScroll(icon_win_y + UHeight);
     return True;
   }
@@ -1433,7 +1434,7 @@ void ParseOptions(void)
       unsigned width,height;
 
       if(strlen(&tline[0])>1){
-	if (mystrncasecmp(tline,CatString3("*", MyName,
+	if (strncasecmp(tline,CatString3("*", MyName,
 					  "Geometry"),Clength+9)==0){
 	  tmp = &tline[Clength+9];
 	  while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
@@ -1452,7 +1453,7 @@ void ParseOptions(void)
 	    xneg = 1;
 	  if (flags & YNegative)
 	    yneg = 1;
-	} else if (mystrncasecmp(tline,CatString3("*", MyName,
+	} else if (strncasecmp(tline,CatString3("*", MyName,
 						"MaxIconSize"),Clength+12)==0){
 	  tmp = &tline[Clength+12];
 	  while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
@@ -1470,63 +1471,68 @@ void ParseOptions(void)
 	    max_icon_width += 4;
 	  }
 	}else if
-	  (mystrncasecmp(tline,CatString3("*",MyName,"Font"),Clength+5)==0)
+	  (strncasecmp(tline,CatString3("*",MyName,"Font"),Clength+5)==0)
 	  CopyString(&font_string,&tline[Clength+5]);
 	else if
-	  (mystrncasecmp(tline,CatString3("*",MyName,"IconFore"),Clength+9)==0)
+	  (strncasecmp(tline,CatString3("*",MyName,"IconFore"),Clength+9)==0)
 	  CopyString(&IconFore,&tline[Clength+9]);
 	else if
-	  (mystrncasecmp(tline,CatString3("*",MyName,"IconBack"),Clength+9)==0)
+	  (strncasecmp(tline,CatString3("*",MyName,"IconBack"),Clength+9)==0)
 	    CopyString(&IconBack,&tline[Clength+9]);
 	else if
-	  (mystrncasecmp(tline,CatString3("*",MyName,"IconHiFore"),Clength+11)==0)
+	  (strncasecmp(tline,CatString3("*",MyName,"IconHiFore"),Clength+11)==0)
 	  CopyString(&ActIconFore,&tline[Clength+11]);
 	else if
-	  (mystrncasecmp(tline,CatString3("*",MyName,"IconHiBack"),Clength+11)==0)
+	  (strncasecmp(tline,CatString3("*",MyName,"IconHiBack"),Clength+11)==0)
 	  CopyString(&ActIconBack,&tline[Clength+11]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Fore"),Clength+5)==0) 
 	  CopyString(&Fore,&tline[Clength+5]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Back"),Clength+5)==0) 
 	  CopyString(&Back,&tline[Clength+5]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Pixmap"),Clength+7)==0) 
 	  CopyString(&IconwinPixmapFile,&tline[Clength+7]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Padding"),Clength+8)==0) 
 	  interval = max(0,atoi(&tline[Clength+8]));
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
+					      "FrameWidth"),Clength+11)==0){ 
+	  sscanf(&tline[Clength+11], "%d %d", &margin1, &margin2);
+	  margin1 = max(0, margin1);
+	  margin2 = max(0, margin2);
+	}else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Lines"),Clength+6)==0)
 	  Lines = max(1,atoi(&tline[Clength+6]));
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "SBWidth"),Clength+8)==0) 
 	  bar_width = max(5,atoi(&tline[Clength+8]));
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 						"Placement"),Clength+10)==0)
 	  parseplacement(&tline[Clength+10]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "SetWMIconSize"),Clength+14)==0)
 	  local_flags |= SETWMICONSIZE;
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "HilightFocusWin"),Clength+16)==0)
 	  m_mask |= M_FOCUS_CHANGE;
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Resolution"),Clength+11)==0){
 	  tmp = &tline[Clength+11];
 	  while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
 	    tmp++;
-	  if (mystrncasecmp(tmp, "Desk", 4) == 0){
+	  if (strncasecmp(tmp, "Desk", 4) == 0){
 	    m_mask |= M_NEW_DESK;
 	    local_flags |= CURRENT_ONLY;
 	  }
-	}else if (mystrncasecmp(tline,CatString3("*",MyName,
+	}else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Mouse"),Clength+6)==0)
 	  parsemouse(&tline[Clength + 6]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "Key"),Clength+4)==0)
 	  parsekey(&tline[Clength + 4]);
-	else if (mystrncasecmp(tline,CatString3("*",MyName,
+	else if (strncasecmp(tline,CatString3("*",MyName,
 					      "SortIcons"),Clength+10)==0){
 	  tmp = &tline[Clength+10];
 	  while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
@@ -1535,31 +1541,31 @@ void ParseOptions(void)
 	    sortby = ICONNAME;
 	    return;
 	  }
-	  if (mystrncasecmp(tmp, "WindowName", 10) == 0)
+	  if (strncasecmp(tmp, "WindowName", 10) == 0)
 	    sortby = WINDOWNAME;
-	  else if (mystrncasecmp(tmp, "IconName", 8) == 0)
+	  else if (strncasecmp(tmp, "IconName", 8) == 0)
 	    sortby = ICONNAME;
-	  else if (mystrncasecmp(tmp, "ResClass", 8) == 0)
+	  else if (strncasecmp(tmp, "ResClass", 8) == 0)
 	    sortby = RESCLASS;
-	  else if (mystrncasecmp(tmp, "ResName", 7) == 0)
+	  else if (strncasecmp(tmp, "ResName", 7) == 0)
 	    sortby = RESNAME;
-	}else if (mystrncasecmp(tline,CatString3("*",MyName,
+	}else if (strncasecmp(tline,CatString3("*",MyName,
 					      "HideSC"),Clength+7)==0){
 	  tmp = &tline[Clength+7];
 	  while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
 	    tmp++;
-	  if (mystrncasecmp(tmp, "Horizontal", 10) == 0)
+	  if (strncasecmp(tmp, "Horizontal", 10) == 0)
 	    local_flags |= HIDE_H;
-	  else if (mystrncasecmp(tmp, "Vertical", 8) == 0)
+	  else if (strncasecmp(tmp, "Vertical", 8) == 0)
 	    local_flags |= HIDE_V;
-	}else if (mystrncasecmp(tline,CatString3("*",MyName,
+	}else if (strncasecmp(tline,CatString3("*",MyName,
 					      ""),Clength+1)==0)
 	  parseicon(&tline[Clength + 1]);
-	else if (mystrncasecmp(tline,"IconPath",8)==0)
+	else if (strncasecmp(tline,"IconPath",8)==0)
 	  CopyString(&iconPath,&tline[8]);
-	else if (mystrncasecmp(tline,"PixmapPath",10)==0)
+	else if (strncasecmp(tline,"PixmapPath",10)==0)
 	  CopyString(&pixmapPath,&tline[10]);
-	else if (mystrncasecmp(tline,"ClickTime",9)==0)
+	else if (strncasecmp(tline,"ClickTime",9)==0)
 	  ClickTime = atoi(&tline[9]);
       }
       GetConfigLine(fd,&tline);
@@ -1630,22 +1636,22 @@ void parseplacement(char *tline)
 
   sscanf(tline, "%s %s", p, s);
   
-  if (mystrncasecmp(p, "Left", 4) == 0)
+  if (strncasecmp(p, "Left", 4) == 0)
     primary = LEFT;
-  else if (mystrncasecmp(p, "Right", 5) == 0)
+  else if (strncasecmp(p, "Right", 5) == 0)
     primary = RIGHT;	
-  else if (mystrncasecmp(p, "Top", 3) == 0)
+  else if (strncasecmp(p, "Top", 3) == 0)
     primary = TOP;
-  else if (mystrncasecmp(p, "Bottom", 6) == 0)
+  else if (strncasecmp(p, "Bottom", 6) == 0)
     primary = BOTTOM;
 
-  if (mystrncasecmp(s, "Left", 4) == 0)
+  if (strncasecmp(s, "Left", 4) == 0)
     secondary = LEFT;
-  else if (mystrncasecmp(s, "Right", 5) == 0)
+  else if (strncasecmp(s, "Right", 5) == 0)
     secondary = RIGHT;	
-  else if (mystrncasecmp(s, "Top", 3) == 0)
+  else if (strncasecmp(s, "Top", 3) == 0)
     secondary = TOP;
-  else if (mystrncasecmp(s, "Bottom", 6) == 0)
+  else if (strncasecmp(s, "Bottom", 6) == 0)
     secondary = BOTTOM;
 }
 
@@ -1666,11 +1672,11 @@ void parsemouse(char *tline)
   end = tline;
   while((!isspace(*end))&&(*end!='\n')&&(*end!=0))
     end++;
-  if (mystrncasecmp(start, "1", 1) == 0)
+  if (strncasecmp(start, "1", 1) == 0)
     f->mouse = Button1;
-  else if (mystrncasecmp(start, "2", 1) == 0)
+  else if (strncasecmp(start, "2", 1) == 0)
     f->mouse = Button2;
-  else if (mystrncasecmp(start, "3", 1) == 0)
+  else if (strncasecmp(start, "3", 1) == 0)
     f->mouse = Button3;
   /* click or doubleclick */
   tline = end;
@@ -1681,9 +1687,9 @@ void parsemouse(char *tline)
   end = tline;
   while((!isspace(*end))&&(*end!='\n')&&(*end!=0))
     end++;
-  if (mystrncasecmp(start, "Click", 5) == 0)
+  if (strncasecmp(start, "Click", 5) == 0)
     f->type = CLICK;
-  else if (mystrncasecmp(start, "DoubleClick", 11) == 0)
+  else if (strncasecmp(start, "DoubleClick", 11) == 0)
     f->type = DOUBLE_CLICK;
 
   /* actions */
@@ -1875,6 +1881,8 @@ void process_message(unsigned long type, unsigned long *body)
 	    olddesk = tmp->desk;
 	    tmp->desk = body[7];
 	    if (olddesk == CurrentDesk || tmp->desk == CurrentDesk){
+	      if (tmp->desk == CurrentDesk && sortby != UNSORT)
+		SortItem(NULL);
 	      num_icons = AdjustIconWindows();
 	      GetIconwinSize(&diffx, &diffy);
 	      if (diffy && (primary == BOTTOM || secondary == BOTTOM))
@@ -2022,7 +2030,8 @@ void process_message(unsigned long type, unsigned long *body)
   case M_NEW_DESK:
     if (CurrentDesk != body[0]){
       CurrentDesk = body[0];
-      if (body[0] != 10000){ /* "magic" number used in FvwmPager */
+      if (body[0] != 10000 && ready){ /* 10000 is a "magic" number used in FvwmPager */
+	if (sortby != UNSORT)
 	SortItem(NULL);
 	num_icons = AdjustIconWindows();
 	GetIconwinSize(&diffx, &diffy);
@@ -2110,7 +2119,7 @@ int AdjustIconWindows(void)
   tmp = Head;
 
   while(tmp != NULL){
-    if (desk_cond(tmp))
+    if (desk_cond(tmp) && tmp->IconWin != None)
       AdjustIconWindow(tmp, i++);
     tmp = tmp->next;
   }
@@ -2319,14 +2328,16 @@ struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
 
 Bool SortItem(struct icon_info *item)
 {
-  struct icon_info *tmp1, *a, *b, *tmp2;
+  struct icon_info *tmp1=Head, *a, *b, *tmp2;
+
+  if (tmp1 == NULL)
+    return False;
 
   if (item != NULL && 
       ((itemcmp(item->prev, item) <= 0) &&  
        (itemcmp(item->next, item) >= 0)))
     return False;
 
-  tmp1 = Head;
   while (tmp1->next != NULL){
     tmp2 = MinItem(tmp1);
     if (tmp1 == tmp2){
