@@ -20,7 +20,7 @@
  * as long as the copyright is kept intact. */
 
 
-#include "../../configure.h"
+#include <FVWMconfig.h>
 
 #include <stdio.h>
 #include <signal.h>
@@ -47,8 +47,8 @@
 #include <X11/cursorfont.h>
 
 #include "../../fvwm/module.h"
-#include "../../version.h"
-#include "../../libs/fvwmlib.h"  /* for pixmaps routines */
+#include <fvwm/version.h>
+#include <fvwm/fvwmlib.h>  /* for pixmaps routines */
 
 #include "FvwmTaskBar.h"
 #include "ButtonArray.h"
@@ -57,7 +57,7 @@
 #include "Mallocs.h"
 #include "Goodies.h"
 #include "Start.h"
-
+#include "GoodyLoadable.h"
 
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
 #define SomeButtonDown(a) ((a)&Button1Mask||(a)&Button2Mask||(a)&Button3Mask)
@@ -100,8 +100,10 @@ int  win_width    = 5,
      button_width = DEFAULT_BTN_WIDTH,
      Clength,
      ButPressed   = -1,
+     ButReleased  = -1,
      Checked      = 0,
-     AlarmSet     = 0;
+     BelayHide    = False,
+     AlarmSet     = NOT_SET;
 
 int UpdateInterval = 30;
 
@@ -117,18 +119,26 @@ char *ClickAction[3] = { DEFAULT_CLICK1, DEFAULT_CLICK2, DEFAULT_CLICK3 },
      *font_string    = "fixed",
      *selfont_string = NULL;
 
-int UseSkipList  = 0,
-    UseIconNames = 0,
-    adjust_flag  = False,
-    AutoStick    = False,
-    HighlightFocus = False;
+int UseSkipList    = False,
+    UseIconNames   = False,
+    ShowTransients = False,
+    adjust_flag    = False,
+    AutoStick      = False,
+    AutoHide       = False,
+    HighlightFocus = False,
+    DeskOnly       = False;
 
 unsigned int ScreenWidth, ScreenHeight;
 
-int NRows, RowHeight;
+int NRows, RowHeight, Midline;
+
+#define COUNT_LIMIT    10000
+long DeskNumber;               /* Added by Balaji R */
+int  First = 1;
+int  Count = 0;
 
 /* Imported from Goodies */
-extern int stwin_width, goodies_width;
+extern int stwin_width, goodies_width, icons_offset;
 extern TipStruct Tip;
 
 /* Imported from Start */
@@ -137,15 +147,19 @@ extern char *StartPopup;
 
 Colormap PictureCMap;
 
-char *IconPath   = NULL,
-     *PixmapPath = NULL;
+#if defined(FVWM_ICONDIR)
+char *IconPath   = FVWM_ICONDIR;
+#else
+char *IconPath   = NULL;
+#endif
+char *PixmapPath = NULL;
 
 /******************************************************************************
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
 ******************************************************************************/
-void main(int argc, char **argv)
+int main(int argc, char **argv)
 {
   char *temp, *s;
 
@@ -164,11 +178,16 @@ void main(int argc, char **argv)
   /* Open the console for messages */
   OpenConsole();
 
+/*#define DEBUG_TASKBAR*/
+#if defined(DEBUG_TASKBAR)
+  /* sys/types.h unistd.h */
+  ConsoleMessage("Started taskbar (pid=%d) ...\n", getpid());
+  system("sleep 20");
+#endif /* DEBUG_TASKBAR */
+
   if((argc != 6)&&(argc != 7)) {
-    fprintf(stderr,"%s Version %s should only be executed by fvwm!\n",Module,
-	    VERSION);
-    ConsoleMessage("%s Version %s should only be executed by fvwm!\n",Module,
-		   VERSION);
+    fprintf(stderr, "%s: Should be started only by fvwm!\n", Module);
+    ConsoleMessage("Should be started only by fvwm!\n");
     exit(1);
   }
 
@@ -180,14 +199,14 @@ void main(int argc, char **argv)
   signal (SIGPIPE, DeadPipe);
   signal (SIGALRM, Alarm);
 
+  /* Parse the config file */
+  InitList(&swallowed);
+  ParseConfig();
+
   SetMessageMask(Fvwm_fd,M_ADD_WINDOW | M_CONFIGURE_WINDOW | M_DESTROY_WINDOW |
 		 M_WINDOW_NAME | M_ICON_NAME | M_RES_NAME | M_DEICONIFY | M_ICONIFY |
 		 M_END_WINDOWLIST | M_FOCUS_CHANGE | M_FUNCTION_END | M_CONFIG_INFO |
-		 M_TITLE_ICON);
-
-  /* Parse the config file */
-  InitList(&swallowed);
-  ParseConfig(argv[3]);
+		 M_MINI_ICON | M_SCROLLREGION | M_NEW_DESK);
 
   /* Setup the XConnection */
   StartMeUp();
@@ -209,6 +228,8 @@ void main(int argc, char **argv)
 
   /* Receive all messages from Fvwm */
   EndLessLoop();
+  
+  return 0;
 }
 
 /******************************************************************************
@@ -220,22 +241,29 @@ void EndLessLoop()
   struct timeval tv;
 
   while(1) {
+    if (XPending(dpy)) {
+      LoopOnEvents();
+      continue;
+    }
     FD_ZERO(&readset);
     FD_SET(Fvwm_fd[1], &readset);
     FD_SET(x_fd, &readset);
-    XPending(dpy);
     tv.tv_sec  = 0;
     tv.tv_usec = 0;
 #ifdef __hpux
     if (!select(fd_width, (int *)&readset, NULL, NULL, &tv)) {
       while(1) {
+        if (XPending(dpy)) {
+          LoopOnEvents();
+          continue;
+        }
         FD_ZERO(&readset);
         FD_SET(Fvwm_fd[1], &readset);
         FD_SET(x_fd, &readset);
-        XPending(dpy);
 
         tv.tv_sec  = UpdateInterval;
         tv.tv_usec = 0;
+
         if (select(fd_width, (int *)&readset, NULL, NULL, &tv) <= 0) 
           DrawGoodies();
         else
@@ -245,10 +273,13 @@ void EndLessLoop()
 #else
     if (!select(fd_width, &readset, NULL, NULL, &tv)) {
       while(1) {
+        if (XPending(dpy)) {
+          LoopOnEvents();
+          continue;
+        }
         FD_ZERO(&readset);
         FD_SET(Fvwm_fd[1], &readset);
         FD_SET(x_fd, &readset);
-        XPending(dpy);
 
         tv.tv_sec  = UpdateInterval;
         tv.tv_usec = 0;
@@ -280,9 +311,7 @@ void EndLessLoop()
 ******************************************************************************/
 void ReadFvwmPipe()
 {
-  int count,total,count2=0,body_length;
   unsigned long header[HEADER_SIZE],*body;
-  char *cbody;
   if(ReadFvwmPacket(Fvwm_fd[1],header,&body) > 0)
     {
       ProcessMessage(header[1],body);
@@ -297,12 +326,13 @@ void ReadFvwmPipe()
 ******************************************************************************/
 void ProcessMessage(unsigned long type,unsigned long *body)
 {
-  int redraw=-1;
+  int redraw = -1;
   int i;
   long flags;
-  char *name,*string;
+  long Desk;
+  char *string;
   Picture p;
-  
+
   switch(type) {
   case M_FOCUS_CHANGE:
     i = FindItem(&windows, body[0]);
@@ -310,6 +340,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       if (ItemFlags(&windows, body[0]) & ICONIFIED) i = -1;
       RadioButton(&buttons, i, BUTTON_BRIGHT);
       ButPressed = i;
+      ButReleased = -1;
       redraw = 0;
     }
     break;
@@ -317,54 +348,73 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   case M_ADD_WINDOW:
   case M_CONFIGURE_WINDOW:
     /* Matched only at startup when default border width and
-       actual border width differ */
+       actual border width differ. Don't assign win_width here so 
+       Window redraw and rewarping gets handled by XEvent
+       ConfigureNotify code. */
+    if (!ShowTransients && (body[8] & TRANSIENT)) break;
     if (body[0] == win) {
       if (win_border != (int)body[10]) {
-	win_y -= win_border;
 	win_x = win_border = (int)body[10];
-	win_y += win_border;
-	win_width = ScreenWidth - (win_border << 1);
-	XMoveResizeWindow(dpy, win, win_x, win_y, win_width, win_height);
-	AdjustWindow(win_width, win_height);
-        if (Tip.open) ShowTipWindow(0);
-        Tip.type = NO_TIP;
-	redraw = 1;
-      } else if ( (int)body[3] != 0 ||
-		 ((int)body[4] != 0 &&
-		  (int)body[4] != ScreenHeight 
-                                  - win_height - (win_border<<1)) &&
-		 AutoStick) {
-	if ((int)body[4] > (ScreenHeight>>1))
-	  win_y = ScreenHeight - win_height - win_border;
-	else
-	  win_y = win_border;
-	XMoveWindow(dpy, win, win_x, win_y);
-        if (Tip.open) ShowTipWindow(0);
-        Tip.type = NO_TIP;
+
+        if (win_y > Midline)
+          win_y = ScreenHeight - (AutoHide ? 1 : win_height + win_border);
+        else 
+          win_y = AutoHide ? 1-win_height : win_border;
+
+        XMoveResizeWindow(dpy, win, win_x, win_y,
+                          ScreenWidth-(win_border<<1), win_height);
+
       }
       break;
     }
-    if (FindItem(&windows,body[0]) != -1) break;
+
+    if ((i=FindItem(&windows,body[0])) != -1) {
+      if (GetDeskNumber(&windows,i,&Desk) && DeskOnly) {
+        if (DeskNumber != Desk && DeskNumber == body[7]) {
+          /* window moving to current desktop */
+          AddButton(&buttons, ItemName(&windows,i), 
+                    GetItemPicture(&windows,i), BUTTON_UP, i);
+          redraw = 1;
+        }
+        if (DeskNumber != body[7] && DeskNumber == Desk) {
+          /* window moving to another desktop */
+          RemoveButton(&buttons, i);
+          redraw = 1;
+        }
+      }
+      UpdateItemFlagsDesk(&windows, body[0], body[8], body[7]);
+      break;
+    }
+
     if (!(body[8] & WINDOWLISTSKIP) || !UseSkipList) {
-      AddItem(&windows, body[0], body[8]);
+      AddItem(&windows, body[0], body[8], body[7], Count++);
+      if (Count > COUNT_LIMIT) Count = 0;
     }
     break;
 
   case M_DESTROY_WINDOW:
-    if ((i = DeleteItem(&windows, body[0])) == -1) break;
+    if ((i = FindItem(&windows, body[0])) == -1) break;
     if (FindItem(&swallowed, body[0]) != -1) break;
-    RemoveButton(&buttons, i);
-    redraw = 1;
+    if (GetDeskNumber(&windows, i, &Desk)) {
+      DeleteItem(&windows, body[0]);
+      if (Desk == DeskNumber || !DeskOnly) {
+        RemoveButton(&buttons, i); /* what about sticky windows? */
+                                   /* there are problems when they are
+                                      deleted from another desktop! */
+        redraw = 1;
+      }
+    }
     break;
 
-  case M_TITLE_ICON:
+  case M_MINI_ICON:
     if ((i = FindItem(&windows, body[0])) == -1) break;
+    p.picture = body[1];
+    p.mask    = body[2];
+    p.width   = body[3];
+    p.height  = body[4];
+    p.depth   = body[5];
+    UpdateItemPicture(&windows, i, &p);
     if (UpdateButton(&buttons, i, NULL, DONT_CARE) != -1) {
-      p.picture = body[1];
-      p.mask    = body[2];
-      p.width   = body[3];
-      p.height  = body[4];
-      p.depth   = body[5];
       UpdateButtonPicture(&buttons, i, &p);
       redraw = 0;
     }
@@ -373,20 +423,24 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   case M_WINDOW_NAME:
   case M_ICON_NAME:
     if ((type == M_ICON_NAME   && !UseIconNames) || 
-	(type == M_WINDOW_NAME &&  UseIconNames)) break;
+        (type == M_WINDOW_NAME &&  UseIconNames)) break;
     string = (char *) &body[3];
     if ((i = FindNameItem(&swallowed, string)) != -1) {
       if (ItemIndexFlags(&swallowed, i) == F_NOT_SWALLOWED) {
-	Swallow(body);
-	break;
+        Swallow(body);
+        break;
       }
     }
     if ((i = UpdateItemName(&windows, body[0], (char *)&body[3])) == -1) 
       break;
     if (UpdateButton(&buttons, i, string, DONT_CARE) == -1)
       {
-      AddButton(&buttons, string, NULL, BUTTON_UP);
-      redraw = 1;
+      if (GetDeskNumber(&windows, i, &Desk) == 0) return; /* ?? */
+      if (!DeskOnly || Desk == DeskNumber)
+        {
+        AddButton(&buttons, string, NULL, BUTTON_UP, i);
+        redraw = 1;
+        }
       }
     else
       redraw = 0;
@@ -399,9 +453,9 @@ void ProcessMessage(unsigned long type,unsigned long *body)
     if (type == M_DEICONIFY && !(flags & ICONIFIED)) break;
     if (type == M_ICONIFY   &&   flags & ICONIFIED) break;
     flags ^= ICONIFIED;
-    if (type == M_ICONIFY && i == ButPressed) {
+    if (type == M_ICONIFY && i == ButReleased) {
       RadioButton(&buttons, -1, BUTTON_UP);
-      ButPressed = -1;
+      ButReleased = ButPressed = -1;
       redraw = 0;
     }
     UpdateItemFlags(&windows, body[0], flags);
@@ -413,10 +467,27 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 
   case M_FUNCTION_END:
     StartButtonUpdate(NULL, BUTTON_UP);
+    
+    if (AutoHide && !BelayHide) /* We don't want the taskbar to hide */
+      SetAlarm(HIDE_TASK_BAR);  /* after a Focus or Iconify function */
+
     redraw = 0;
     break;
 
+  /* Added a new Fvwm Event because scrolling regions interfere 
+     with EnterNotify event when taskbar is hidden. */
+  case M_SCROLLREGION:
+    if (AutoHide && ((win_y <  Midline && body[1] < 4) ||
+                     (win_y >= Midline && body[1] > ScreenHeight-4)))
+      RevealTaskBar();
+    break;
+
   case M_NEW_DESK:
+    DeskNumber = body[0];
+    if (!First && DeskOnly)
+      redraw_buttons();
+    else
+      First = 0;
     break;
 
   case M_NEW_PAGE:
@@ -427,6 +498,22 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   if (redraw >= 0) RedrawWindow(redraw);
 }
 
+void redraw_buttons()
+{
+  Item *item;
+
+  while (buttons.head)
+    RemoveButton(&buttons, buttons.head->count);
+
+  for (item=windows.head; item; item=item->next)
+    if (DeskNumber == item->Desk || ((item->flags & STICKY) &&
+				     !(item->flags & ICONIFIED)))
+      AddButton(&buttons, item->name, &(item->p), BUTTON_UP, item->count);
+    
+  RedrawWindow(1);
+}
+
+
 /******************************************************************************
   SendFvwmPipe - Send a message back to fvwm 
     Based on SendInfo() from FvwmIdent:
@@ -472,7 +559,7 @@ void SendFvwmPipe(char *message, unsigned long window)
  **********************************************************************/
 void DeadPipe(int nonsense)
 {
-  ConsoleMessage("received SIGPIPE signal: exiting...\n");
+  ConsoleMessage("Received SIGPIPE signal, exiting...\n");
   ShutMeDown(1);
 }
 
@@ -519,6 +606,7 @@ void ConsoleMessage(char *fmt, ...)
     filep = stderr;
   else
     filep = console;
+  fprintf(filep, "%s: ", Module);
   va_start(args, fmt);
   vfprintf(filep, fmt, args);
   va_end(args);
@@ -545,111 +633,134 @@ int OpenConsole()
     Based on part of main() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
 ******************************************************************************/
-void ParseConfig(char *file)
+void ParseConfig()
 {
-  char line[256];
   char *tline;
   char *str;
-  FILE *ptr;
   int i, j;
+  unsigned long header[HEADER_SIZE];
+  unsigned long type;
+  unsigned long *body;
+  int running;
+  int n;
    
-  ptr = fopen(file,"r");
-  if (ptr == (FILE *)NULL) {
-    ConsoleMessage("Couldn't read configuration file (%s)...\n", file);
-    return;
-  }
+  /* Request config info */
+  SendFvwmPipe("Send_ConfigInfo",0);
 
-  tline = fgets(line,(sizeof line)-1,ptr);
+  /* 
+   * We use the "running" variable to count 2 M_FUNCTION_END messages,
+   * one from the preceding SetMessageMask, and one from our Send_ConfigInfo.
+   */
+  running = 1;
 
-  while (tline != (char *)0) {
-    while (isspace(*tline))tline++;
-    if (strlen(tline)>1 && tline[0] != '#') {
-      if(mystrncasecmp(tline, CatString3(Module, "Font",""),Clength+4)==0)
-	CopyString(&font_string,&tline[Clength+4]);
-      else if(mystrncasecmp(tline, CatString3(Module, "SelFont",""),Clength+7)==0)
-	CopyString(&selfont_string,&tline[Clength+7]);
-      else if(mystrncasecmp(tline,CatString3(Module,"Fore",""), Clength+4)==0)
-	CopyString(&ForeColor,&tline[Clength+4]);
-      else if(mystrncasecmp(tline,CatString3(Module, "Geometry",""), Clength+8)==0) {
-	str = &tline[Clength+9];
-	while(((isspace(*str))&&(*str != '\n'))&&(*str != 0))	str++;
-	str[strlen(str)-1] = 0;
-	UpdateString(&geometry,str);
-      } else if(mystrncasecmp(tline,CatString3(Module, "Back",""), Clength+4)==0)
-	CopyString(&BackColor,&tline[Clength+4]);
-      else if(mystrncasecmp(tline,CatString3(Module, "Action",""), Clength+6)==0)
-	LinkAction(&tline[Clength+6]);
-      else if(mystrncasecmp(tline,CatString3(Module, "UseSkipList",""),
-			    Clength+11)==0) UseSkipList=1;
-      else if(mystrncasecmp(tline,CatString3(Module, "AutoStick",""),
+  while (running) {
+    n = ReadFvwmPacket(Fvwm_fd[1],header,&body);
+    if (n < 0) ShutMeDown(2);
+    if (n == 0) continue;
+
+    type = header[1];
+
+    if (type == M_FUNCTION_END) {
+      running--;
+    } else if (type == M_CONFIG_INFO) {
+
+      tline = (char *)&body[3];
+
+      while (isspace(*tline))tline++;
+      if (strlen(tline)>1 && tline[0] != '#') {
+        if(strncasecmp(tline, CatString3(Module, "Font",""),Clength+4)==0)
+  	  CopyString(&font_string,&tline[Clength+4]);
+        else if(strncasecmp(tline, CatString3(Module, "SelFont",""),Clength+7)==0)
+	  CopyString(&selfont_string,&tline[Clength+7]);
+        else if(strncasecmp(tline,CatString3(Module,"Fore",""), Clength+4)==0)
+	  CopyString(&ForeColor,&tline[Clength+4]);
+        else if(strncasecmp(tline,CatString3(Module, "Geometry",""), Clength+8)==0) {
+	  str = &tline[Clength+9];
+	  while(((isspace(*str))&&(*str != '\n'))&&(*str != 0))	str++;
+	  str[strlen(str)-1] = 0;
+	  UpdateString(&geometry,str);
+        } else if(strncasecmp(tline,CatString3(Module, "Back",""), Clength+4)==0)
+	  CopyString(&BackColor,&tline[Clength+4]);
+        else if(strncasecmp(tline,CatString3(Module, "Action",""), Clength+6)==0)
+	  LinkAction(&tline[Clength+6]);
+        else if(strncasecmp(tline,CatString3(Module, "UseSkipList",""),
+			    Clength+11)==0) UseSkipList=True;
+        else if(strncasecmp(tline,CatString3(Module, "AutoStick",""),
 			    Clength+9)==0) AutoStick=True;
-      else if(mystrncasecmp(tline,CatString3(Module, "UseIconNames",""),
-			    Clength+12)==0) UseIconNames=1;
-      else if(mystrncasecmp(tline,CatString3(Module, "UpdateInterval",""),
+        else if(strncasecmp(tline,CatString3(Module, "AutoHide",""),
+                            Clength+8)==0) { AutoHide=True; AutoStick=True; }
+        else if(strncasecmp(tline,CatString3(Module, "UseIconNames",""),
+			    Clength+12)==0) UseIconNames=True;
+        else if(strncasecmp(tline,CatString3(Module, "ShowTransients",""),
+			    Clength+14)==0) ShowTransients=True;
+        else if(strncasecmp(tline,CatString3(Module, "DeskOnly",""),
+                            Clength+8)==0) DeskOnly=True;
+        else if(strncasecmp(tline,CatString3(Module, "UpdateInterval",""),
                             Clength+14)==0)
                                UpdateInterval=atoi(&tline[Clength+14]);
-      else if(mystrncasecmp(tline,CatString3(Module, "HighlightFocus",""),
+        else if(strncasecmp(tline,CatString3(Module, "HighlightFocus",""),
                             Clength+14)==0) HighlightFocus=True;
-      else if(mystrncasecmp(tline,CatString3(Module, "SwallowModule",""),
+        else if(strncasecmp(tline,CatString3(Module, "SwallowModule",""),
 			    Clength+13)==0) {
 	
-	/* tell fvwm to launch the module for us */
-	str = safemalloc(strlen(&tline[Clength+13]) + 6);
-	sprintf(str, "Module %s",&tline[Clength+13]);
-	ConsoleMessage("Trying to: %s", str);
-	SendFvwmPipe(str, 0);
+	  /* tell fvwm to launch the module for us */
+	  str = safemalloc(strlen(&tline[Clength+13]) + 8);
+	  sprintf(str, "Module %s",&tline[Clength+13]);
+	  ConsoleMessage("Trying to: %s", str);
+	  SendFvwmPipe(str, 0);
 	
-	/* Remember the anticipated window's name for swallowing */
-	i = 4;	      
-	while((str[i] != 0)&&
-	      (str[i] != '"'))
-	  i++;
-	j = ++i;
-	while((str[i] != 0)&&
-	      (str[i] != '"'))
-	  i++;
-	if (i > j) {
-	  str[i] = 0;
-	  ConsoleMessage("Looking for window: [%s]\n", &str[j]);
-	  AddItemName(&swallowed, &str[j], F_NOT_SWALLOWED);
-	}	
-	free(str);
-      } else if(mystrncasecmp(tline,CatString3(Module, "Swallow",""),
+	  /* Remember the anticipated window's name for swallowing */
+	  i = 4;	      
+	  while((str[i] != 0)&&
+	        (str[i] != '"'))
+	    i++;
+	  j = ++i;
+	  while((str[i] != 0)&&
+	        (str[i] != '"'))
+	    i++;
+	  if (i > j) {
+	    str[i] = 0;
+	    ConsoleMessage("Looking for window: [%s]\n", &str[j]);
+	    AddItemName(&swallowed, &str[j], F_NOT_SWALLOWED);
+	  }	
+	  free(str);
+        } else if(strncasecmp(tline,CatString3(Module, "Swallow",""),
 			      Clength+7)==0) {
 	
-	/* tell fvwm to Exec the process for us */
-	str = safemalloc(strlen(&tline[Clength+7]) + 6);
-	sprintf(str, "Exec %s",&tline[Clength+7]);
-	ConsoleMessage("Trying to: %s", str);
-	SendFvwmPipe(str, 0);
+	  /* tell fvwm to Exec the process for us */
+	  str = safemalloc(strlen(&tline[Clength+7]) + 6);
+	  sprintf(str, "Exec %s",&tline[Clength+7]);
+	  ConsoleMessage("Trying to: %s", str);
+	  SendFvwmPipe(str, 0);
 	
-	/* Remember the anticipated window's name for swallowing */
-	i = 4;	      
-	while((str[i] != 0)&&
-	      (str[i] != '"'))
+	  /* Remember the anticipated window's name for swallowing */
+	  i = 4;	      
+	  while((str[i] != 0)&&
+	        (str[i] != '"'))
+	    i++;
+	  j = ++i;
+	  while((str[i] != 0)&&
+	        (str[i] != '"'))
 	  i++;
-	j = ++i;
-	while((str[i] != 0)&&
-	      (str[i] != '"'))
-	  i++;
-	if (i > j) {
-	  str[i] = 0;
-	  ConsoleMessage("Looking for window: [%s]\n", &str[j]);
-	  AddItemName(&swallowed, &str[j], F_NOT_SWALLOWED);
-	}	
-	free(str);
-      } else if(mystrncasecmp(tline,"ButtonWidth",11) == 0) {
-	button_width = atoi(&tline[11]);
-      } else if(mystrncasecmp(tline,"IconPath",8) == 0) {
-	CopyString(&IconPath, &tline[8]);
-      } else if(mystrncasecmp(tline,"PixmapPath",10) == 0) {
-	CopyString(&PixmapPath, &tline[10]);
-      } else {
-	GoodiesParseConfig(tline, Module);
-	StartButtonParseConfig(tline, Module);
+	  if (i > j) {
+	    str[i] = 0;
+	    ConsoleMessage("Looking for window: [%s]\n", &str[j]);
+	    AddItemName(&swallowed, &str[j], F_NOT_SWALLOWED);
+	  }	
+	  free(str);
+        } else if(strncasecmp(tline,"ButtonWidth",11) == 0) {
+	  button_width = atoi(&tline[11]);
+        } else if(strncasecmp(tline,"IconPath",8) == 0) {
+	  CopyString(&IconPath, &tline[8]);
+        } else if(strncasecmp(tline,"PixmapPath",10) == 0) {
+	  CopyString(&PixmapPath, &tline[10]);
+        } else {
+	  GoodiesParseConfig(tline, Module);
+	  StartButtonParseConfig(tline, Module);
+        }
       }
-      }
-    tline = fgets(line,(sizeof line)-1,ptr);
+    }
+    free(body);
   }
 }
 
@@ -691,48 +802,64 @@ void Swallow(unsigned long *body) {
   Alarm - Handle a SIGALRM - used to implement timeout events
 ******************************************************************************/
 void Alarm(int nonsense) {
-  ShowTipWindow(1);
-  AlarmSet = 0;
+
+  switch(AlarmSet) {
+
+  case SHOW_TIP:
+    ShowTipWindow(1);
+    break;
+
+  case HIDE_TASK_BAR:
+    HideTaskBar();
+    break;
+  }
+
+  AlarmSet = NOT_SET;
   signal (SIGALRM, Alarm);
 }
 
 /******************************************************************************
   CheckForTip - determine when to popup the tip window
 ******************************************************************************/
+extern int GLtip;
+
 void CheckForTip(int x, int y) {
-  int  num, bx, by;
+  int  num, bx, by, trunc, i_k;
   char *name;
 
   if (MouseInStartButton(x, y)) {
     if (Tip.type != START_TIP) PopupTipWindow(3, 0, "Click here to start");
     Tip.type = START_TIP;
-  }
-  else if (MouseInMail(x, y)) {
-    if (Tip.type != MAIL_TIP) CreateMailTipWindow();
-    Tip.type = MAIL_TIP;
+  } else {
+    if ((i_k = LoadableSeeMouse(x, y)) != 0) {
+      if (!IsLoadableTip(Tip.type)) GLtip = -1;
+      Tip.type = CreateLoadableTipWindow(i_k);
+    } else {
+      num = LocateButton(&buttons, x, y, &bx, &by, &name, &trunc);
+      if (num != -1 && trunc) {
+        if ((Tip.type != num)  || 
+            (Tip.text == NULL) || (strcmp(name, Tip.text) != 0))
+          PopupTipWindow(bx+3, by, name);
+        Tip.type = num;
+      } else {
+        Tip.type = NO_TIP;
+      }
     }
-  else if (MouseInClock(x, y)) {
-    if (Tip.type != DATE_TIP) CreateDateWindow();
-    Tip.type = DATE_TIP;
-    }
-  else if ((num = LocateButton(&buttons, x, y, &bx, &by, &name)) != -1) {
-    if ((Tip.type != num)  || 
-        (Tip.text == NULL) || (strcmp(name, Tip.text) != 0))
-      PopupTipWindow(bx+3, by, name);
-    Tip.type = num;
   }
-  else Tip.type = NO_TIP; 
 
   if (Tip.type != NO_TIP) {
     if (!AlarmSet && !Tip.open) {
       alarm(1);
       AlarmSet = 1;
     }
+    if (AlarmSet != SHOW_TIP && !Tip.open) 
+      SetAlarm(SHOW_TIP);
   } else {
     if (AlarmSet) {
       alarm(0);
       AlarmSet = 0;
     }
+    ClearAlarm();
     if (Tip.open) ShowTipWindow(0);
   }
 }
@@ -743,11 +870,13 @@ void CheckForTip(int x, int y) {
 ******************************************************************************/
 void LoopOnEvents()
 {
-  int  num;
+  int  num = -1;
+  int  i_k;
   char tmp[100];
   XEvent Event;
-  int x, x1, y, y1, redraw;
+  int x, y, redraw;
   static unsigned long lasttime = 0L;
+  Time time = 0L;
 
   while(XPending(dpy)) {
     redraw = -1;
@@ -757,6 +886,8 @@ void LoopOnEvents()
       case ButtonRelease:
         num = WhichButton(&buttons, Event.xbutton.x, Event.xbutton.y);
         if (num != -1) {
+          ButReleased = ButPressed; /* Avoid race fvwm pipe */
+          BelayHide = True; /* Don't AutoHide when function ends */
           SendFvwmPipe(ClickAction[Event.xbutton.button-1], 
                        ItemID(&windows, num));
           redraw = 0;
@@ -767,6 +898,7 @@ void LoopOnEvents()
           if (num != -1) SendFvwmPipe("Focus 0", ItemID(&windows, num));
         }
         ButPressed = -1;
+	time = Event.xbutton.time;
         break;
 
       case ButtonPress:
@@ -774,21 +906,23 @@ void LoopOnEvents()
 	if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y)) {
 	  StartButtonUpdate(NULL, BUTTON_DOWN);
           x = win_x;
-          if (win_y == win_border) {
-            /* bar at top of the screen */
-            y = win_border + RowHeight;
+          if (win_y < Midline) {
+            /* bar in top half of the screen */
+            y = win_y + RowHeight;
           } else {
-            /* bar at top of the screen */
-            y = -(win_height+win_border);
+            /* bar in bottom of the screen */
+            y = win_y - ScreenHeight;
           }
           sprintf(tmp,"Popup %s %d %d", StartPopup, x, y);
-          SendFvwmPipe(tmp, ItemID(&windows, num));
-        } else if (MouseInMail(Event.xbutton.x, Event.xbutton.y)) {
-           HandleMailClick(Event);
+          SendFvwmPipe(tmp, ItemID(&windows, 0));
+        } else if ((i_k=LoadableSeeMouse(Event.xbutton.x, Event.xbutton.y))!=0){
+	  HandleLoadableClick(Event, i_k);
 	} else {
           num = WhichButton(&buttons, Event.xbutton.x, Event.xbutton.y);
-	  UpdateButton(&buttons, num, NULL, (ButPressed == num) ?
-                                             BUTTON_BRIGHT : BUTTON_DOWN);
+          UpdateButton(&buttons, num, NULL, (ButPressed == num) ?
+                                              BUTTON_BRIGHT : BUTTON_DOWN);
+ 
+/*	  UpdateButton(&buttons, num, NULL, BUTTON_DOWN);*/
           ButPressed = num;
 	}
 /*      else { / * Move taskbar * /
@@ -799,6 +933,7 @@ void LoopOnEvents()
         }
 */
         redraw = 0;
+	time = Event.xbutton.time;
         break;
 
       case Expose:
@@ -807,14 +942,18 @@ void LoopOnEvents()
             redraw = 0;
           else
             redraw = 1;
+	/* time = ; */
         break;
 
       case ClientMessage:
         if ((Event.xclient.format==32) && (Event.xclient.data.l[0]==wm_del_win))
           ShutMeDown(0);
+	/* time = ; */
         break;
 
       case EnterNotify:
+        if (AutoHide) RevealTaskBar();
+
         if (Event.xcrossing.mode != NotifyNormal) break;
         num = WhichButton(&buttons, Event.xcrossing.x, Event.xcrossing.y);
         if (!HighlightFocus) {
@@ -832,42 +971,48 @@ void LoopOnEvents()
             SendFvwmPipe("Focus 0", ItemID(&windows, num));
         }
 
-        CheckForTip(Event.xmotion.x, Event.xmotion.y);
+        CheckForTip(Event.xcrossing.x, Event.xcrossing.y);
+	time = Event.xcrossing.time;
         break;
 
       case LeaveNotify:
-        if (AlarmSet) {
-          alarm(0);
-          AlarmSet = 0;
-        }
+        ClearAlarm();
         if (Tip.open) ShowTipWindow(0);
 
         if (Event.xcrossing.mode != NotifyNormal) break;
+
+        if (AutoHide) SetAlarm(HIDE_TASK_BAR);
+
         if (!HighlightFocus) {
           if (SomeButtonDown(Event.xcrossing.state)) {
-            if (ButPressed != -1)
-              {
+            if (ButPressed != -1) {
               RadioButton(&buttons, -1, BUTTON_UP);
               ButPressed = -1;
               redraw = 0;
-              }
+            }
+          } else {
+            if (ButReleased != -1) {
+              RadioButton(&buttons, -1, BUTTON_UP);
+              ButReleased = -1;
+              redraw = 0;
+            }
           }
         }
+	time = Event.xcrossing.time;
         break;
 
       case MotionNotify:
         num = WhichButton(&buttons, Event.xmotion.x, Event.xmotion.y);
         if (!HighlightFocus) {
-          if (SomeButtonDown(Event.xmotion.state)) {
-            if (num != ButPressed) {
-              if (num != -1) {
-                RadioButton(&buttons, num, BUTTON_DOWN);
-                ButPressed = num;
-                redraw = 0;
-              } else {
-                ButPressed = -1;
-              }
+          if (SomeButtonDown(Event.xmotion.state) && num != ButPressed) {
+            if (num != -1) {
+              RadioButton(&buttons, num, BUTTON_DOWN);
+              ButPressed = num;
+            } else {
+              RadioButton(&buttons, num, BUTTON_UP);
+              ButPressed = -1;
             }
+            redraw = 0;
           }
         } else {
           if (num != -1 && num != ButPressed)
@@ -875,25 +1020,36 @@ void LoopOnEvents()
         }
 
         CheckForTip(Event.xmotion.x, Event.xmotion.y);
+	time = Event.xmotion.time;
         break;
       
       case ConfigureNotify:
         if ((Event.xconfigure.width != win_width ||
 	     Event.xconfigure.height != win_height)) {
 	  AdjustWindow(Event.xconfigure.width, Event.xconfigure.height);
+          if (AutoStick) WarpTaskBar(win_y);
 	  redraw = 1;
         }
+        else if (Event.xconfigure.x != win_x || Event.xconfigure.y != win_y) {
+          if (AutoStick) {
+            WarpTaskBar(Event.xconfigure.y);
+          } else {
+            win_x = Event.xconfigure.x;
+            win_y = Event.xconfigure.y;
+          }
+        }
+	/* time = ; */
         break;
     }
 
-    if (Event.xkey.time - lasttime > UpdateInterval*1000L) {
+    if (redraw >= 0) RedrawWindow(redraw);
+
+    if (time - lasttime > UpdateInterval*1000L) {
       DrawGoodies();
-      lasttime = Event.xkey.time;
+      lasttime = time;
     }
   }
 
-  if (redraw >= 0) RedrawWindow(redraw);
-
 } 
 
 /***********************************
@@ -930,13 +1086,13 @@ void LinkAction(char *string)
 char *temp;
   temp=string;
   while(isspace(*temp)) temp++;
-  if(mystrncasecmp(temp, "Click1", 6)==0)
+  if(strncasecmp(temp, "Click1", 6)==0)
     CopyString(&ClickAction[0],&temp[6]);
-  else if(mystrncasecmp(temp, "Click2", 6)==0)
+  else if(strncasecmp(temp, "Click2", 6)==0)
     CopyString(&ClickAction[1],&temp[6]);
-  else if(mystrncasecmp(temp, "Click3", 6)==0)
+  else if(strncasecmp(temp, "Click3", 6)==0)
     CopyString(&ClickAction[2],&temp[6]);
-  else if(mystrncasecmp(temp, "Enter", 5)==0)
+  else if(strncasecmp(temp, "Enter", 5)==0)
     CopyString(&EnterAction,&temp[5]);
 }
 
@@ -948,9 +1104,7 @@ void StartMeUp()
    XSizeHints hints;
    XGCValues gcval;
    unsigned long gcmask;
-   unsigned int dummy1,dummy2;
-   int x,y,ret,count;
-   Window dummyroot,dummychild;
+   int ret;
 
    if (!(dpy = XOpenDisplay(""))) {
       fprintf(stderr,"%s: can't open display %s", Module,
@@ -962,20 +1116,22 @@ void StartMeUp()
    Root = RootWindow(dpy, screen);
    d_depth = DefaultDepth(dpy, screen);
 
-   ScreenWidth = XDisplayWidth(dpy, screen);
+   ScreenWidth  = XDisplayWidth(dpy, screen);
    ScreenHeight = XDisplayHeight(dpy, screen);
+
+   Midline = (int) (ScreenHeight >> 1);
    
    if (selfont_string == NULL) selfont_string = font_string;
 
    if ((ButtonFont = XLoadQueryFont(dpy, font_string)) == NULL) {
      if ((ButtonFont = XLoadQueryFont(dpy, "fixed")) == NULL) {
-       ConsoleMessage("Couldn't load fixed font. Exiting!\n");
+       ConsoleMessage("Couldn't load fixed font, exiting...\n");
        exit(1);
      }
    }
    if ((SelButtonFont = XLoadQueryFont(dpy, selfont_string)) == NULL) {
      if ((SelButtonFont = XLoadQueryFont(dpy, "fixed")) == NULL) {
-       ConsoleMessage("Couldn't load fixed font. Exiting!\n");
+       ConsoleMessage("Couldn't load fixed font, exiting...\n");
        exit(1);
      }
    }
@@ -989,17 +1145,18 @@ void StartMeUp()
    win_height = RowHeight;
    win_width = ScreenWidth - (win_border << 1);
 
-   ret=XParseGeometry(geometry, &hints.x, &hints.y,
-		      (unsigned int *)&hints.width,
-		      (unsigned int *)&hints.height);
+   ret = XParseGeometry(geometry, &hints.x, &hints.y,
+	 	        (unsigned int *)&hints.width,
+		        (unsigned int *)&hints.height);
+
    if (ret & YNegative)
-     hints.y = ScreenHeight - win_height - win_border;
-   else
-     hints.y = win_border;
+     hints.y = ScreenHeight - (AutoHide ? 1 : win_height + (win_border<<1));
+   else 
+     hints.y = AutoHide ? 1 - win_height : 0;
    
    hints.flags=USPosition|PPosition|USSize|PSize|PResizeInc|
      PWinGravity|PMinSize|PMaxSize|PBaseSize;
-   hints.x = win_border;
+   hints.x           = 0;
    hints.width       = win_width;
    hints.height      = RowHeight;
    hints.width_inc   = win_width;
@@ -1304,15 +1461,112 @@ void ConstrainSize (XSizeHints *hints, int *widthp, int *heightp)
   return;
 }
 
+/***********************************************************************
+ WarpTaskBar -- Enforce AutoStick feature
+ ***********************************************************************/
+void WarpTaskBar(int y) {
+  win_x = win_border;
+
+  if (!AutoHide) {
+    if (y < Midline)
+      win_y = win_border;
+    else
+      win_y = (int)ScreenHeight - win_height - win_border;
+
+    XMoveWindow(dpy, win, win_x, win_y);
+  }
+  if (AutoHide) SetAlarm(HIDE_TASK_BAR);
+
+  /* Prevent oscillations caused by race with 
+     time delayed TaskBarHide().  Is there any way
+     to prevent these Xevents from being sent
+     to the server in the first place? */
+  PurgeConfigEvents(); 
+}
+
+/***********************************************************************
+ RevealTaskBar -- Make taskbar fully visible
+ ***********************************************************************/
+void RevealTaskBar() {
+  int new_win_y;
+
+  ClearAlarm();
+
+  if (win_y < Midline) {
+    new_win_y = win_border;
+    for (; win_y<=new_win_y; win_y+=2) 
+	XMoveWindow(dpy, win, win_x, win_y);    
+  } else {
+      new_win_y = (int)ScreenHeight - win_height - win_border;
+      for (; win_y>=new_win_y; win_y-=2) 
+	  XMoveWindow(dpy, win, win_x, win_y);
+  }
+  
+  win_y = new_win_y;
+  XMoveWindow(dpy, win, win_x, win_y);
+  BelayHide = False; 
+}
+
+/***********************************************************************
+ HideTaskbar -- Make taskbar partially visible
+ ***********************************************************************/
+void HideTaskBar() {
+  int new_win_y;
+
+  ClearAlarm();
+
+  if (win_y < Midline) {
+    new_win_y = 1 - win_height;
+    for (; win_y>=new_win_y; --win_y)
+      XMoveWindow(dpy, win, win_x, win_y);
+  } else {
+    new_win_y = (int)ScreenHeight - 1;
+    for (; win_y<=new_win_y; ++win_y)
+      XMoveWindow(dpy, win, win_x, win_y);
+  }
+
+  win_y = new_win_y;
+  /* XMoveWindow(dpy, win, win_x, win_y); */
+}
+
+/***********************************************************************
+ SetAlarm -- Schedule a timeout event
+ ************************************************************************/
+void SetAlarm(int event) {
+  AlarmSet = event;
+  alarm(1);
+}
+
+/***********************************************************************
+ ClearAlarm -- Disable timeout events
+ ************************************************************************/
+void ClearAlarm(void) {
+  if(AlarmSet) {
+    AlarmSet = NOT_SET;
+    alarm(0);
+  }
+}
+
+/***********************************************************************
+ PurgeConfigEvents -- Wait for and purge ConfigureNotify events.
+ ************************************************************************/
+void PurgeConfigEvents(void) {
+  XEvent Event;
+
+  XPeekEvent(dpy, &Event);
+  while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event));
+}
+
 /************************************************************************
   X Error Handler
 ************************************************************************/
-XErrorHandler ErrorHandler(Display *d, XErrorEvent *event)
-  {
+XErrorHandler ErrorHandler(Display *d, XErrorEvent *event) {
   char errmsg[256];
 
   XGetErrorText(d, event->error_code, errmsg, 256);
-  ConsoleMessage("%s failed request: %s\n", Module, errmsg);
+  ConsoleMessage("Failed request: %s\n", errmsg);
   ConsoleMessage("Major opcode: 0x%x, resource id: 0x%x\n",
                   event->request_code, event->resourceid);
-  }
+  return 0;
+}
+
