@@ -16,7 +16,22 @@
  * No guarantees or warantees or anything
  * are provided or implied in any way whatsoever. Use this program at your
  * own risk. Permission to use this program for any purpose is given,
- * as long as the copyright is kept intact. */
+ * as long as the copyright is kept intact.
+
+ * Modifications Done to Add Pixmaps, focus highlighting and Current Desk Only
+ * By Don Mahurin, 1996, Some of this Code was taken from FvwmTaskBar
+
+ * Bug Notes:(Don Mahurin)
+
+ * Moving a window doesnt send M_CONFIGURE, as I thought it should. Desktop
+ * for that button is not updated.
+
+ * There seems to be something incosistant about Resizing and the value given
+ * back as win_x, win_y.  SEE RESIZE_BUG_HACK for my work around. 
+ 
+ * Don Mahurin (dmahurin@donet.com)
+
+*/
 
 #define TRUE 1
 #define FALSE 
@@ -33,7 +48,7 @@
 #define YES "Yes"
 #define NO  "No"
 
-#include "../../configure.h"
+#include <FVWMconfig.h>
 
 #include <stdio.h>
 #include <signal.h>
@@ -59,7 +74,7 @@
 #include <X11/cursorfont.h>
 
 #include "../../fvwm/module.h"
-#include "../../version.h"
+#include <fvwm/version.h>
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
 #include "List.h"
@@ -102,14 +117,28 @@ char *ClickAction[3]={"Iconify -1,Raise","Iconify","Lower"},*EnterAction,
 char *font_string = "fixed";
 int UseSkipList=0,Anchor=1,UseIconNames=0,LeftJustify=0,TruncateLeft=1;
 
+long CurrentDesk = 0;
+int ShowCurrentDesk = 0;
+
+int RESIZE_BUG_HACK = 0;
+
+int ItemCountD(List *list )
+{
+	if(!ShowCurrentDesk)
+		return ItemCount(list);
+	/* else */
+	return ItemCountDesk(list, CurrentDesk);
+
+}
+
 /******************************************************************************
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
 ******************************************************************************/
-void main(int argc, char **argv)
+int main(int argc, char **argv)
 {
-char *temp, *s;
+  char *temp, *s;
 
   /* Save the program name for error messages and config parsing */
   temp = argv[0];
@@ -135,7 +164,7 @@ char *temp, *s;
   }
 
 
-  if ((argc==7)&&(!mystrcasecmp(argv[6],"Transient"))) Transient=1;
+  if ((argc==7)&&(!strcasecmp(argv[6],"Transient"))) Transient=1;
 
   Fvwm_fd[0] = atoi(argv[1]);
   Fvwm_fd[1] = atoi(argv[2]);
@@ -147,8 +176,11 @@ char *temp, *s;
 
   /* Setup the XConnection */
   StartMeUp();
+  XSetErrorHandler((XErrorHandler) ErrorHandler);
 
-  InitArray(&buttons,0,0,win_width,fontheight+6);
+  InitPictureCMap(dpy, Root);
+
+  InitArray(&buttons,0,0,win_width, fontheight+6);
   InitList(&windows);
 
   fd_width = GetFdWidth();
@@ -162,6 +194,8 @@ char *temp, *s;
 	      M_NEW_DESK| M_NEW_PAGE);*/
   /* Recieve all messages from Fvwm */
   EndLessLoop();
+
+  return 0;
 }
 
 /******************************************************************************
@@ -225,31 +259,66 @@ void ReadFvwmPipe()
 ******************************************************************************/
 void ProcessMessage(unsigned long type,unsigned long *body)
 {
-int redraw=0,i;
-long flags;
-char *name,*string;
-static current_focus=-1;
+  int redraw=0,i;
+  long flags;
+  char *name,*string;
+  static current_focus=-1;
 
-  switch(type) {
+  Picture p;
+
+  switch(type)
+  {
     case M_ADD_WINDOW:
     case M_CONFIGURE_WINDOW:
-      if (body[0]==win) {
+      if (body[0] == win)
+      {
         win_x=(int)body[3];
         win_y=(int)body[4];
+
         win_title=(int)body[9];
         win_border=(int)body[10];
-        if (win_border!=0) win_border++;
       }
-      if (FindItem(&windows,body[0])!=-1) break;
+      if ((i = FindItem(&windows,body[0]))!=-1) 
+      { 
+/*	printf("does it go here on MOVE %ld, %ld ?\n", i, CurrentDesk);
+*/
+	if(UpdateItemDesk(&windows, i, body[7]) > 0)
+        {
+/*         printf("why doesnt it go here on MOVE ?\n");
+*/
+          AdjustWindow();
+          RedrawWindow(1);
+        }
+	break;
+      }
+
       if (!(body[8]&WINDOWLISTSKIP) || !UseSkipList)
-        AddItem(&windows,body[0],body[8]);
+        AddItem(&windows,body[0],body[8], body[7] /* desk */);
       break;
     case M_DESTROY_WINDOW:
       if ((i=DeleteItem(&windows,body[0]))==-1) break;
       RemoveButton(&buttons,i);
-      if (WindowIsUp) AdjustWindow();
+      if (WindowIsUp)
+        AdjustWindow();
+
       redraw=1;
       break;
+
+  case M_MINI_ICON:
+    if ((i=FindItem(&windows,body[0]))==-1) break;
+
+    if (UpdateButton(&buttons,i,NULL,-1)!=-1)
+    {
+      p.picture = body[1];
+      p.mask = body[2];
+      p.width = body[3];
+      p.height = body[4];
+      p.depth = body[5];
+      UpdateButtonPicture(&buttons, i, &p);
+      redraw = 0;
+    }
+    break;
+
     case M_WINDOW_NAME:
     case M_ICON_NAME:
       if ((type==M_ICON_NAME && !UseIconNames) || 
@@ -259,8 +328,9 @@ static current_focus=-1;
       name=makename(string,ItemFlags(&windows,body[0]));
       if (UpdateButton(&buttons,i,name,-1)==-1)
       {
-        AddButton(&buttons,name,1);
+	AddButton(&buttons, name, NULL, 1);
 	UpdateButtonSet(&buttons,i,ItemFlags(&windows,body[0])&ICONIFIED?1:0);
+        UpdateButtonDesk(&buttons,i,ItemDesk(&windows, body[0]));
       }
       free(name);
       if (WindowIsUp) AdjustWindow();
@@ -282,6 +352,9 @@ static current_focus=-1;
       free(name);
       break;
     case M_FOCUS_CHANGE:
+
+/* old coded, before Task bar code spliced in
+
       i=FindItem(&windows,body[0]);
       if (i==current_focus) break;
       if (current_focus!=-1)
@@ -292,12 +365,33 @@ static current_focus=-1;
       current_focus=i;
       if (current_focus!=-1)
 	if (UpdateButtonSet(&buttons,current_focus,2)!=-1) redraw=1;
+* end of old code */
+
+/* Code copied Straight from TaskBar */
+
+    if ((i=FindItem(&windows,body[0]))!=-1)
+    {
+	    flags=ItemFlags(&windows,body[0]);
+ 	    UpdateItemFlags(&windows,body[0],flags);
+    }
+    RadioButton(&buttons,i);
+    redraw = 1;
+    break;
+
       break;
     case M_END_WINDOWLIST:
       if (!WindowIsUp) MakeMeWindow();
       redraw=1;
       break;
     case M_NEW_DESK:
+	CurrentDesk = body[0];
+	if(ShowCurrentDesk)
+	{
+	  RESIZE_BUG_HACK = 1;
+	  AdjustWindow();
+	  RESIZE_BUG_HACK = 0;
+	  RedrawWindow(1);
+        }
       break;
     case M_NEW_PAGE:
       break;
@@ -313,13 +407,16 @@ static current_focus=-1;
 ******************************************************************************/
 void SendFvwmPipe(char *message,unsigned long window)
 {
-int w;
-char *hold,*temp,*temp_msg;
+  int w;
+  char *hold,*temp,*temp_msg;
+
   hold=message;
 
-  while(1) {
+  while(1)
+  {
     temp=strchr(hold,',');
-    if (temp!=NULL) {
+    if (temp!=NULL)
+    {
       temp_msg=malloc(temp-hold+1);
       strncpy(temp_msg,hold,(temp-hold));
       temp_msg[(temp-hold)]='\0';
@@ -356,10 +453,13 @@ void DeadPipe(int nonsense)
 ******************************************************************************/
 void WaitForExpose(void)
 {
-XEvent Event;
-  while(1) {
+  XEvent Event;
+
+  while(1)
+  {
     XNextEvent(dpy,&Event);
-    if (Event.type==Expose) {
+    if (Event.type==Expose)
+    {
       if (Event.xexpose.count==0) break;
     }
   }
@@ -370,7 +470,7 @@ XEvent Event;
 ******************************************************************************/
 void RedrawWindow(int force)
 {
-  DrawButtonArray(&buttons,force);
+  DrawButtonArray(&buttons, force);
   if (XQLength(dpy) && !force) LoopOnEvents();
 }
 
@@ -380,8 +480,9 @@ void RedrawWindow(int force)
 void ConsoleMessage(char *fmt, ...)
 {
 #ifndef NO_CONSOLE
-va_list args;
-FILE *filep;
+  va_list args;
+  FILE *filep;
+
   if (console==NULL) filep=stderr;
   else filep=console;
   va_start(args,fmt);
@@ -418,39 +519,41 @@ void ParseConfig()
     {
       if(strlen(tline)>1) 
 	{
-	  if(mystrncasecmp(tline, CatString3(Module, "Font",""),Clength+4)==0)
+	  if(strncasecmp(tline, CatString3(Module, "Font",""),Clength+4)==0)
 	    CopyString(&font_string,&tline[Clength+4]);
-	  else if(mystrncasecmp(tline,CatString3(Module,"Fore",""), Clength+4)==0)
+	  else if(strncasecmp(tline,CatString3(Module,"Fore",""), Clength+4)==0)
 	    CopyString(&ForeColor[0],&tline[Clength+4]);
-	  else if(mystrncasecmp(tline,CatString3(Module,"IconFore",""), Clength+8)==0)
+	  else if(strncasecmp(tline,CatString3(Module,"IconFore",""), Clength+8)==0)
 	    CopyString(&ForeColor[1],&tline[Clength+8]);
-	  else if(mystrncasecmp(tline,CatString3(Module,"FocusFore",""), Clength+9)==0)
+	  else if(strncasecmp(tline,CatString3(Module,"FocusFore",""), Clength+9)==0)
 	    CopyString(&ForeColor[2],&tline[Clength+9]);
-	  else if(mystrncasecmp(tline,CatString3(Module, "Geometry",""), Clength+8)==0)
+	  else if(strncasecmp(tline,CatString3(Module, "Geometry",""), Clength+8)==0)
 	    CopyString(&geometry,&tline[Clength+8]);
-	  else if(mystrncasecmp(tline,CatString3(Module, "Back",""), Clength+4)==0)
+	  else if(strncasecmp(tline,CatString3(Module, "Back",""), Clength+4)==0)
 	    CopyString(&BackColor[0],&tline[Clength+4]);
-	  else if(mystrncasecmp(tline,CatString3(Module,"IconBack",""), Clength+8)==0)
+	  else if(strncasecmp(tline,CatString3(Module,"IconBack",""), Clength+8)==0)
 	    CopyString(&BackColor[1],&tline[Clength+8]);
-	  else if(mystrncasecmp(tline,CatString3(Module,"FocusBack",""), Clength+9)==0)
+	  else if(strncasecmp(tline,CatString3(Module,"FocusBack",""), Clength+9)==0)
 	    CopyString(&BackColor[2],&tline[Clength+9]);
-	  else if(mystrncasecmp(tline,CatString3(Module, "NoAnchor",""),
+	  else if(strncasecmp(tline,CatString3(Module, "NoAnchor",""),
 				Clength+8)==0) Anchor=0;
-	  else if(mystrncasecmp(tline,CatString3(Module, "Action",""), Clength+6)==0)
+	  else if(strncasecmp(tline,CatString3(Module, "Action",""), Clength+6)==0)
 	    LinkAction(&tline[Clength+6]);
-	  else if(mystrncasecmp(tline,CatString3(Module, "UseSkipList",""),
+	  else if(strncasecmp(tline,CatString3(Module, "UseSkipList",""),
 				Clength+11)==0) UseSkipList=1;
-	  else if(mystrncasecmp(tline,CatString3(Module, "UseIconNames",""),
+	  else if(strncasecmp(tline,CatString3(Module, "UseIconNames",""),
 				Clength+12)==0) UseIconNames=1;
-	  else if(mystrncasecmp(tline,CatString3(Module, "LeftJustify",""),
+	  else if(strncasecmp(tline,CatString3(Module, "ShowCurrentDesk",""),
+				Clength+15)==0) ShowCurrentDesk=1;
+	  else if(strncasecmp(tline,CatString3(Module, "LeftJustify",""),
 				Clength+11)==0) LeftJustify=1;
-	  else if(mystrncasecmp(tline,CatString3(Module, "TruncateLeft",""),
+	  else if(strncasecmp(tline,CatString3(Module, "TruncateLeft",""),
 				Clength+12)==0) TruncateLeft=1;
-	  else if(mystrncasecmp(tline,CatString3(Module, "TruncateRight",""),
+	  else if(strncasecmp(tline,CatString3(Module, "TruncateRight",""),
 				Clength+13)==0) TruncateLeft=0;
-          else if(mystrncasecmp(tline,CatString3(Module, "MinWidth",""),
+          else if(strncasecmp(tline,CatString3(Module, "MinWidth",""),
                                 Clength+8)==0) MinWidth=atoi(&tline[Clength+8]);
-          else if(mystrncasecmp(tline,CatString3(Module, "MaxWidth",""),
+          else if(strncasecmp(tline,CatString3(Module, "MaxWidth",""),
                                 Clength+8)==0) MaxWidth=atoi(&tline[Clength+8]);
 	}
       GetConfigLine(Fvwm_fd,&tline);
@@ -462,17 +565,19 @@ void ParseConfig()
 ******************************************************************************/
 void LoopOnEvents()
 {
-int num;
-char buffer[10];
-XEvent Event;
-Window dummyroot,dummychild;
-int x,x1,y,y1;
-unsigned int dummy1;
+  int num;
+  char buffer[10];
+  XEvent Event;
+  Window dummyroot,dummychild;
+  int x,x1,y,y1;
+  unsigned int dummy1;
 
-  if (Transient && !Checked) {
+  if (Transient && !Checked)
+  {
     XQueryPointer(dpy,win,&dummyroot,&dummychild,&x1,&y1,&x,&y,&dummy1);
     num=WhichButton(&buttons,x,y);
-    if (num!=-1) {
+    if (num!=-1)
+    {
       Pressed=1;
       ButPressed=num;
       SwitchButton(&buttons,num);
@@ -480,14 +585,18 @@ unsigned int dummy1;
     Checked=1;
   }
 
-  while(XPending(dpy)) {
+  while(XPending(dpy))
+  {
     XNextEvent(dpy,&Event);
 
-    switch(Event.type) {
+    switch(Event.type)
+    {
       case ButtonRelease:
-        if (Pressed) {
+        if (Pressed)
+        {
           num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-          if (num!=-1) {
+          if (num!=-1)
+          {
             SendFvwmPipe(ClickAction[(Transient) ? 0:Event.xbutton.button-1],
               ItemID(&windows,num));
             SwitchButton(&buttons,num);
@@ -499,7 +608,8 @@ unsigned int dummy1;
         break;
       case ButtonPress:
         num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-        if (num != -1) {
+        if (num != -1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
         } else ButPressed=-1;
@@ -511,7 +621,8 @@ unsigned int dummy1;
         break;
       case KeyPress:
         num=XLookupString(&Event.xkey,buffer,10,NULL,0);
-        if (num==1) {
+        if (num==1)
+        {
           if (buffer[0]=='q' || buffer[0]=='Q') ShutMeDown(0);
           else if (buffer[0]=='i' || buffer[0]=='I') PrintList(&windows);
           else if (buffer[0]=='b' || buffer[0]=='B') PrintButtons(&buttons);
@@ -523,7 +634,8 @@ unsigned int dummy1;
       case EnterNotify:
         if (!SomeButtonDown(Event.xcrossing.state)) break;
         num=WhichButton(&buttons,Event.xcrossing.x,Event.xcrossing.y);
-        if (num!=-1) {
+        if (num!=-1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
         } else ButPressed=-1;
@@ -539,10 +651,13 @@ unsigned int dummy1;
         num=WhichButton(&buttons,Event.xmotion.x,Event.xmotion.y);
         if (num==ButPressed) break;
         if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
-        if (num!=-1) {
+        if (num!=-1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
-        } else ButPressed=-1;
+        }
+        else ButPressed=-1;
+
         break;
     }
   }
@@ -553,44 +668,72 @@ unsigned int dummy1;
 ******************************************************************************/
 void AdjustWindow()
 {
-int new_width=0,new_height=0,tw,i,total;
-char *temp;
+  int new_width=0,new_height=0,tw,i,total;
+  char *temp;
 
-  total=ItemCount(&windows);
-  if (!total) {
-    if (WindowIsUp==1) {
+  total = ItemCountD(&windows );
+  if (!total)
+  {
+    if (WindowIsUp==1)
+    {
       XUnmapWindow(dpy,win);
       WindowIsUp=2;
     }
     return;
   }
-  for(i=0;i<total;i++) {
+  for(i=0;i<total;i++)
+  {
     temp=ItemName(&windows,i);
     if(temp != NULL)
-      {
+    {
 	tw=10+XTextWidth(ButtonFont,temp,strlen(temp));
 	tw+=XTextWidth(ButtonFont,"()",2);
+
+	tw+=14; /* for title icon */ /* Magic Number ? */
+
 	new_width=max(new_width,tw);
-      }
+    }
   }
-  new_width=max(new_width,MinWidth);
-  new_width=min(new_width,MaxWidth);
+  new_width=max(new_width, MinWidth);
+  new_width=min(new_width, MaxWidth);
   new_height=(total*(fontheight+6+1)-1);
-  if (WindowIsUp && (new_height!=win_height  || new_width!=win_width)) {
-    if (Anchor) {
+  if (WindowIsUp && (new_height!=win_height  || new_width!=win_width))
+  {
+    if (Anchor)
+    {
       if (win_grav==SouthEastGravity || win_grav==NorthEastGravity)
         win_x-=(new_width-win_width);
+
       if (win_grav==SouthEastGravity || win_grav==SouthWestGravity)
         win_y-=(new_height-win_height);
+
+      if(RESIZE_BUG_HACK)
+      {
+        if (win_grav==SouthEastGravity || win_grav==NorthEastGravity)
+		win_x+= win_border - 2;
+	else
+		win_x-=win_border;
+
+	if (win_grav==SouthEastGravity || win_grav==SouthWestGravity)
+		win_y+=win_border - 2;
+	else	
+		win_y-=win_border;
+      }
+
       XMoveResizeWindow(dpy,win,win_x+win_border,win_y+win_title+win_border,
         new_width,new_height);
-    } else XResizeWindow(dpy,win,new_width,new_height);
+
+    }
+    else
+      XResizeWindow(dpy, win, new_width,new_height);
+
     XSync(dpy,False);
   }
   UpdateArray(&buttons,-1,-1,new_width,-1);
-  if (new_height>0) win_height=new_height;
-  if (new_width>0) win_width=new_width;
-  if (WindowIsUp==2) {
+  if (new_height>0) win_height = new_height;
+  if (new_width>0) win_width = new_width;
+  if (WindowIsUp==2)
+  {
     XMapWindow(dpy,win);
     WindowIsUp=1;
     WaitForExpose();
@@ -619,13 +762,13 @@ void LinkAction(char *string)
 char *temp;
   temp=string;
   while(isspace(*temp)) temp++;
-  if(mystrncasecmp(temp, "Click1", 6)==0)
+  if(strncasecmp(temp, "Click1", 6)==0)
     CopyString(&ClickAction[0],&temp[6]);
-  else if(mystrncasecmp(temp, "Click2", 6)==0)
+  else if(strncasecmp(temp, "Click2", 6)==0)
     CopyString(&ClickAction[1],&temp[6]);
-  else if(mystrncasecmp(temp, "Click3", 6)==0)
+  else if(strncasecmp(temp, "Click3", 6)==0)
     CopyString(&ClickAction[2],&temp[6]);
-  else if(mystrncasecmp(temp, "Enter", 5)==0)
+  else if(strncasecmp(temp, "Enter", 5)==0)
     CopyString(&EnterAction,&temp[5]);
 }
 
@@ -634,15 +777,16 @@ char *temp;
 ******************************************************************************/
 void MakeMeWindow(void)
 {
-XSizeHints hints;
-XGCValues gcval;
-unsigned long gcmask;
-unsigned int dummy1,dummy2;
-int x,y,ret,count;
-Window dummyroot,dummychild;
-int i;
+  XSizeHints hints;
+  XGCValues gcval;
+  unsigned long gcmask;
+  unsigned int dummy1, dummy2;
+  int x, y, ret, count;
+  Window dummyroot, dummychild;
+  int i;
+
   
-  if ((count=ItemCount(&windows))==0 && Transient) ShutMeDown(0);
+  if ((count = ItemCountD(&windows))==0 && Transient) ShutMeDown(0);
   AdjustWindow();
 
   hints.width=win_width;
@@ -652,28 +796,38 @@ int i;
   hints.width_inc=0;
   hints.height_inc=0;
 
-  if (geometry!= NULL) {
+  if (geometry!= NULL)
+  {
     ret=XParseGeometry(geometry,&x,&y,&dummy1,&dummy2);
 
-    if (ret&XValue && ret &YValue) {
+    if (ret&XValue && ret &YValue)
+    {
       hints.x=x;
-      if (ret&XNegative) hints.x+=XDisplayWidth(dpy,screen)-win_width;
+      if (ret&XNegative)
+        hints.x+=XDisplayWidth(dpy,screen)-win_width;
+
       hints.y=y;
-      if (ret&YNegative) hints.y+=XDisplayHeight(dpy,screen)-win_height;
+      if (ret&YNegative)
+        hints.y+=XDisplayHeight(dpy,screen)-win_height;
+
       hints.flags|=USPosition;
     }
 
-    if (ret&XNegative) {
+    if (ret&XNegative)
+    {
       if (ret&YNegative) hints.win_gravity=SouthEastGravity;
       else  hints.win_gravity=NorthEastGravity;
-    } else {
+    }
+    else
+    {
       if (ret&YNegative) hints.win_gravity=SouthWestGravity;
       else  hints.win_gravity=NorthWestGravity;
     }
 
   }
 
-  if (Transient) {
+  if (Transient)
+  {
     XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
     hints.win_gravity=NorthWestGravity;
     hints.flags |= USPosition;
@@ -682,11 +836,15 @@ int i;
   win_x=hints.x;
   win_y=hints.y;
 
+
   for (i = 0; i != MAX_COLOUR_SETS; i++)
-  if(d_depth < 2) {
+  if(d_depth < 2)
+  {
     back[i] = GetColor("white");
     fore[i] = GetColor("black");
-  } else {
+  }
+  else
+  {
     back[i] = GetColor(BackColor[i] == NULL ? BackColor[0] : BackColor[i]);
     fore[i] = GetColor(ForeColor[i] == NULL ? ForeColor[0] : ForeColor[i]);
   }
@@ -699,7 +857,8 @@ int i;
   
   XSetWMNormalHints(dpy,win,&hints);
 
-  if (!Transient) {
+  if (!Transient)
+  {
     XGrabButton(dpy,1,AnyModifier,win,True,GRAB_EVENTS,GrabModeAsync,
       GrabModeAsync,None,None);
     XGrabButton(dpy,2,AnyModifier,win,True,GRAB_EVENTS,GrabModeAsync,
@@ -709,7 +868,9 @@ int i;
     SetMwmHints(MWM_DECOR_ALL|MWM_DECOR_RESIZEH|MWM_DECOR_MAXIMIZE|MWM_DECOR_MINIMIZE,
       MWM_FUNC_ALL|MWM_FUNC_RESIZE|MWM_FUNC_MAXIMIZE|MWM_FUNC_MINIMIZE,
       MWM_INPUT_MODELESS);
-  } else {
+  }
+  else
+  {
     SetMwmHints(0,MWM_FUNC_ALL,MWM_INPUT_MODELESS);
   }
 
@@ -743,13 +904,15 @@ int i;
 
   ChangeWindowName(&Module[1]);
 
-  if (ItemCount(&windows)>0) {
+  if (ItemCountD(&windows) > 0)
+  {
     XMapRaised(dpy,win);
     WaitForExpose();
     WindowIsUp=1;
   } else WindowIsUp=2;
 
-  if (Transient) {
+  if (Transient)
+  {
     if ( XGrabPointer(dpy,win,True,GRAB_EVENTS,GrabModeAsync,GrabModeAsync,
       None,None,CurrentTime)!=GrabSuccess) ShutMeDown(1);
     XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
@@ -763,7 +926,8 @@ int i;
 ******************************************************************************/
 void StartMeUp()
 {
-  if (!(dpy = XOpenDisplay(""))) {
+  if (!(dpy = XOpenDisplay("")))
+  {
     fprintf(stderr,"%s: can't open display %s", Module,
       XDisplayName(""));
     exit (1);
@@ -776,14 +940,14 @@ void StartMeUp()
   ScreenHeight = DisplayHeight(dpy,screen);
   ScreenWidth = DisplayWidth(dpy,screen);
 
-  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL) {
+  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL)
+  {
     if ((ButtonFont=XLoadQueryFont(dpy,"fixed"))==NULL) exit(1);
   }
 
-  fontheight=ButtonFont->ascent+ButtonFont->descent;
+  fontheight = ButtonFont->ascent+ButtonFont->descent;
 
   win_width=XTextWidth(ButtonFont,"XXXXXXXXXXXXXXX",10);
- 
 
 }
 
@@ -853,4 +1017,16 @@ PropMwmHints prop;
     }
 }
 
+/************************************************************************
+  X Error Handler
+************************************************************************/
+XErrorHandler ErrorHandler(Display *d, XErrorEvent *event)
+  {
+  char errmsg[256];
+
+  XGetErrorText(d, event->error_code, errmsg, 256);
+  ConsoleMessage("%s failed request: %s\n", Module, errmsg);
+  ConsoleMessage("Major opcode: 0x%x, resource id: 0x%x\n",
+                  event->request_code, event->resourceid);
+  }
 
